"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FindNftByMintOperationHandler = void 0;
const shared_1 = require("../../../shared");
const utils_1 = require("../../../utils");
const models_1 = require("../models");
const programs_1 = require("../../../programs");
class FindNftByMintOperationHandler extends shared_1.OperationHandler {
    handle(operation) {
        return __awaiter(this, void 0, void 0, function* () {
            const mint = operation.input;
            const metadataPda = yield programs_1.MetadataAccount.pda(mint);
            const editionPda = yield programs_1.MasterEditionAccount.pda(mint);
            const publicKeys = [metadataPda, editionPda];
            const [metadataInfo, editionInfo] = yield this.metaplex.getMultipleAccountsInfo(publicKeys);
            const metadataAccount = metadataInfo ? programs_1.MetadataAccount.fromAccountInfo(metadataInfo) : null;
            const masterEditionAccount = editionInfo
                ? programs_1.MasterEditionAccount.fromAccountInfo(editionInfo)
                : null;
            if (!metadataAccount) {
                // TODO: Custom error.
                throw new Error('Nft not found');
            }
            return new models_1.Nft(metadataAccount, masterEditionAccount, yield this.fetchJsonMetadata(metadataAccount));
        });
    }
    fetchJsonMetadata(metadata) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const uri = (0, utils_1.removeEmptyChars)(metadata.data.data.uri);
                return this.metaplex.storage().downloadJson(uri);
            }
            catch (error) {
                return null;
            }
        });
    }
}
exports.FindNftByMintOperationHandler = FindNftByMintOperationHandler;
//# sourceMappingURL=FindNftByMintOperationHandler.js.map