"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FindNftsByCandyMachineOperationHandler = void 0;
const web3_js_1 = require("@safecoin/web3.js");
const shared_1 = require("../../../shared");
const models_1 = require("../models");
const tokenMetadata_1 = require("../../../programs/tokenMetadata");
class FindNftsByCandyMachineOperationHandler extends shared_1.OperationHandler {
    handle(operation) {
        return __awaiter(this, void 0, void 0, function* () {
            const { input } = operation;
            let firstCreator;
            if (input.v1) {
                firstCreator = input.v1;
            }
            else if (input.v2) {
                // TODO: Refactor when we have a CandyMachine program in the SDK.
                [firstCreator] = yield web3_js_1.PublicKey.findProgramAddress([Buffer.from('candy_machine'), input.v2.toBuffer()], new web3_js_1.PublicKey('cndy3Z4yapfJBmL3ShUp5exZKqR3z33thTzeNMm2gRZ'));
            }
            else {
                // TODO: Custom error.
                throw new Error('Candy Machine address not provided');
            }
            const mintKeys = yield tokenMetadata_1.TokenMetadataProgram.metadataV1Accounts(this.metaplex.connection)
                .selectMint()
                .whereFirstCreator(firstCreator)
                .getDataAsPublicKeys();
            return (shared_1.Postpone.make(() => __awaiter(this, void 0, void 0, function* () { return mintKeys; }))
                // Get PDAs.
                .map((mint) => __awaiter(this, void 0, void 0, function* () {
                return [
                    yield tokenMetadata_1.MetadataAccount.pda(mint),
                    yield tokenMetadata_1.MasterEditionAccount.pda(mint),
                ];
            }))
                // Resolve and flatten PDA promises.
                .asyncPipe((promises) => __awaiter(this, void 0, void 0, function* () { return Promise.allSettled(yield promises); }))
                .flatMap((result) => (result.status === 'fulfilled' ? result.value : []))
                // Feed PDAs into a GetMultipleAccountBuilder.
                .pipe((pdas) => new shared_1.GmaBuilder(this.metaplex.connection, pdas))
                .asyncPipe((gma) => __awaiter(this, void 0, void 0, function* () { return (yield gma).get(); }))
                // Regroup Metadata and MasterEdition accounts.
                .chunk(2)
                // Map Nfts from Metadata and MasterEdition accounts.
                .flatMap(([metadataInfo, editionInfo]) => {
                const metadata = metadataInfo.exists
                    ? tokenMetadata_1.MetadataAccount.fromAccountInfo(metadataInfo)
                    : null;
                const edition = editionInfo.exists
                    ? tokenMetadata_1.MasterEditionAccount.fromAccountInfo(editionInfo)
                    : null;
                return metadata ? [new models_1.Nft(metadata, edition)] : [];
            })
                // Execute the postponed promise.
                .run());
        });
    }
}
exports.FindNftsByCandyMachineOperationHandler = FindNftsByCandyMachineOperationHandler;
//# sourceMappingURL=FindNftsByCandyMachineOperationHandler.js.map